function ArkInventory.MenuMainWaterfallRegister()
	
	local n = string.format( "%1$s v%2$s", ArkInventory.Const.Program.Name, ArkInventory.Const.Program.UIVersion )
	
	if ArkInventory.Const.Program.Beta ~= nil then
		n = string.format( "%s %s(beta %s)", n, RED_FONT_COLOR_CODE, ArkInventory.Const.Program.Beta )
	end
	
	ArkInventory.Global.Waterfall.Tree = {
	
		ArkInventory.MenuMainWaterfallBuildLocation(),
		{
			id = "KEEP_OFFLINE",
			text = ArkInventory.Localise["MOD_MENU_MAIN_OFFLINE"],
		},
		{
			id = "HIDE_BLIZZARD",
			text = ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_BLIZZARD"],
		},
		{
			id = "AUTO_OPEN_CLOSE",
			text = ArkInventory.Localise["MOD_MENU_MAIN_AUTO"],
		}
	}

	ArkInventory.Lib.Waterfall:Register( ArkInventory.Const.Program.Name, "tree", ArkInventory.Global.Waterfall.Tree, "children", ArkInventory.MenuMainWaterfallChildren, "title", n )
 
end

function ArkInventory.MenuMainWaterfallBuildLocation()

	local c = {
			id = "LOCATION_SETTINGS",
			text = ArkInventory.Localise["MOD_MENU_MAIN_SETTINGS"]
	}
	
	for loc_id, loc in ipairs( ArkInventory.Global.Location ) do

		if loc.control then
		
			local n = "LOCATION_SETTINGS_" .. loc_id
		
			local z = {
					id = n,
					text = loc.Name,
					{
						id = n .. "_WINDOW",
						text = ArkInventory.Localise["MOD_MENU_MAIN_WINDOW"],
					},
					{
						id = n .. "_BAR",
						text = ArkInventory.Localise["MOD_MENU_MAIN_BARS"],
					},
					{
						id = n .. "_ITEM",
						text = ArkInventory.Localise["MOD_MENU_MAIN_ITEMS"],
					},
					{
						id = n .. "_SORT",
						text = ArkInventory.Localise["MOD_MENU_MAIN_SORTING"],
					},
					{
						id = n .. "_EMPTY",
						text = ArkInventory.Localise["MOD_MENU_MAIN_EMPTY"],
					},
				}
			
			tinsert( c, z )
			
		end
		
	end

	return c

end

function ArkInventory.MenuMainWaterfallChildren( id )

	if id == nil then
		return ""
	end

	
	local w = ArkInventory.Lib.Waterfall
	
	local cp = ArkInventory.Global.Me
		
	local locations = { }
	for loc_id, loc in ipairs( ArkInventory.Global.Location ) do
		if loc.control then
			locations[string.format( loc_id )] = loc.Name
		end
	end

	local anchorpoints = {
		[string.format( ArkInventory.Const.Anchor.TopRight )] = ArkInventory.Localise["MOD_MENU_GENERIC_TOPRIGHT"],
		[string.format( ArkInventory.Const.Anchor.BottomRight )] = ArkInventory.Localise["MOD_MENU_GENERIC_BOTTOMRIGHT"],
		[string.format( ArkInventory.Const.Anchor.BottomLeft )] = ArkInventory.Localise["MOD_MENU_GENERIC_BOTTOMLEFT"],
		[string.format( ArkInventory.Const.Anchor.TopLeft )] = ArkInventory.Localise["MOD_MENU_GENERIC_TOPLEFT"],
	}

	
	if id == "LOCATION_SETTINGS" then
	
		w:AddControl( "type", "label" )
	

		for loc_id, loc in ipairs( ArkInventory.Global.Location ) do
		
			if loc.control then
			
				w:AddControl(
					"type", "heading",
					"text", loc.Name
				)
		
				w:AddControl(
					"type", "dropdown",
					"list", locations,
					"strict", true,
					"causesRefresh", true,
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SETTINGS"],
					"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SETTINGS_LOCATION_TEXT"], ArkInventory.Global.Location[ArkInventory.db.profile.option.use[loc_id]].Name, loc.Name ),
					"getFunc", function()
						return string.format( ArkInventory.db.profile.option.use[loc_id] )
					end,
					"setFunc", function( set_id )
						ArkInventory.db.profile.option.use[loc_id] = tonumber( set_id )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
					end
				)
	
				w:AddControl( "type", "label" )
		
			end
			
		end
		
		return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_SETTINGS"] )
	
	end
	
	
	if id == "KEEP_OFFLINE" then
	
		w:AddControl( "type", "label" )
	
		for loc_id, loc in ipairs( ArkInventory.Global.Location ) do
			
			if loc.control then
			
				w:AddControl(
					"type", "checkbox",
					"text", loc.Name,
					"tooltipTitle", loc.Name,
					"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_OFFLINE_TEXT"], loc.Name, cp.name ),
					"getFunc", function()
									return ArkInventory.db.account.player.data[cp.player_id].offline[loc_id]
								end,
					"setFunc", function()
									ArkInventory.db.account.player.data[cp.player_id].offline[loc_id] = not ArkInventory.db.account.player.data[cp.player_id].offline[loc_id]
								end
				)
				
			end
	
		end
		
		local setting_name = " [" .. GREEN_FONT_COLOR_CODE .. ArkInventory.Global.Me.player_id .. FONT_COLOR_CODE_CLOSE .. "]"
		return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_OFFLINE"] ) .. setting_name

	end
	
	
	if id == "AUTO_OPEN_CLOSE" then
	
		w:AddControl( "type", "label" )
	
		w:AddControl(
			"type", "heading",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_OPEN"]
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["LOCATION_BANK"],
			"tooltipTitle", ArkInventory.Localise["LOCATION_BANK"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_BANK_OPEN_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.open.bank
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.open.bank = not ArkInventory.db.account.option.auto.open.bank
						end
		)
		
		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["LOCATION_VAULT"],
			"tooltipTitle", ArkInventory.Localise["LOCATION_VAULT"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_VAULT_OPEN_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.open.vault
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.open.vault = not ArkInventory.db.account.option.auto.open.vault
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MAILBOX"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MAILBOX"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MAILBOX_OPEN_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.open.mail
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.open.mail = not ArkInventory.db.account.option.auto.open.mail
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MERCHANT"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MERCHANT"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MAILBOX_OPEN_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.open.merchant
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.open.merchant = not ArkInventory.db.account.option.auto.open.merchant
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_TRADE"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_TRADE"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_TRADE_OPEN_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.open.trade
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.open.trade = not ArkInventory.db.account.option.auto.open.trade
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_AUCTION"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_AUCTION"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_AUCTION_OPEN_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.open.auction
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.open.auction = not ArkInventory.db.account.option.auto.open.auction
						end
		)

		w:AddControl( "type", "label" )
		
		w:AddControl(
			"type", "heading",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_CLOSE"]
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["LOCATION_BANK"],
			"tooltipTitle", ArkInventory.Localise["LOCATION_BANK"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_BANK_CLOSE_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.close.bank
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.close.bank = not ArkInventory.db.account.option.auto.close.bank
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["LOCATION_VAULT"],
			"tooltipTitle", ArkInventory.Localise["LOCATION_VAULT"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_VAULT_CLOSE_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.close.vault
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.close.vault = not ArkInventory.db.account.option.auto.close.vault
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MAILBOX"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MAILBOX"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MAILBOX_CLOSE_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.close.mail
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.close.mail = not ArkInventory.db.account.option.auto.close.mail
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MERCHANT"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MERCHANT"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_MERCHANT_CLOSE_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.close.merchant
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.close.merchant = not ArkInventory.db.account.option.auto.close.merchant
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_TRADE"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_TRADE"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_TRADE_CLOSE_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.close.trade
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.close.trade = not ArkInventory.db.account.option.auto.close.trade
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_AUCTION"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_AUCTION"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_AUCTION_CLOSE_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.close.auction
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.close.auction = not ArkInventory.db.account.option.auto.close.auction
						end
		)

		w:AddControl(
			"type", "checkbox",
			"text", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_COMBAT"],
			"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_AUTO_COMBAT"],
			"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_AUTO_COMBAT_CLOSE_TEXT"], ArkInventory.Const.Program.Name ),
			"getFunc", function()
							return ArkInventory.db.account.option.auto.close.combat
						end,
			"setFunc", function()
							ArkInventory.db.account.option.auto.close.combat = not ArkInventory.db.account.option.auto.close.combat
						end
		)

		return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_AUTO"] )
		
	end
	
	
	if id == "HIDE_BLIZZARD" then
	
		w:AddControl( "type", "label" )
	
		for loc_id, loc in ipairs( ArkInventory.Global.Location ) do
			
			if loc.control then
			
				w:AddControl(
					"type", "checkbox",
					"text", loc.Name,
					"tooltipTitle", loc.Name,
					"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_BLIZZARD_TEXT"], ArkInventory.Const.Program.Name, loc.Name ),
					"getFunc", function()
									return ArkInventory.db.account.option.control[loc_id]
								end,
					"setFunc", function( set_id )
									ArkInventory.db.account.option.control[loc_id] = not ArkInventory.db.account.option.control[loc_id]
									ArkInventory.BlizzardAPIHooks()
								end
				)
				
			end
			
		end
		
		w:AddControl( "type", "label" )

		return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_BLIZZARD"] )
	
	end
	

	if strsub( id, 1, 18 ) == "LOCATION_SETTINGS_" then
	
		local loc_id = strmatch( id, "^LOCATION_SETTINGS_([%d]+)" )
		local key = strmatch( id, "^LOCATION_SETTINGS_" .. loc_id .. "_(.+)" )
	
		if loc_id then

			local loc_id = tonumber( loc_id )
			local setting_name = " [" .. GREEN_FONT_COLOR_CODE .. string.upper( ArkInventory.Global.Location[ArkInventory.db.profile.option.use[loc_id]].Name ) .. FONT_COLOR_CODE_CLOSE .. "]"
			
			if key == "WINDOW" then
			
				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "label",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN"]
				)
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_TITLE"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_TITLE"],
					"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_TITLE_TEXT"], ArkInventory.Const.Program.Name ),
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "framehide", "header" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "framehide", "header" }, not ArkInventory.LocationOptionGet( loc_id, { "framehide", "header" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Init )
					end
				)

				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_CHANGER"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_CHANGER"],
					"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_CHANGER_TEXT"], ArkInventory.Const.Program.Name ),
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "framehide", "changer" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "framehide", "changer" }, not ArkInventory.LocationOptionGet( loc_id, { "framehide", "changer" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Init )
					end
				)

				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_STATUS"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_STATUS"],
					"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_HIDDEN_STATUS_TEXT"], ArkInventory.Const.Program.Name ),
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "framehide", "status" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "framehide", "status" }, not ArkInventory.LocationOptionGet( loc_id, { "framehide", "status" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Init )
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "slider",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_SCALE"] .. " (%)",
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_SCALE"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_SCALE_TEXT"],
					"min", 40,
					"max", 200,
					"step", 5,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "window", "scale" } )
					end,
					"setFunc", function( v )
						local v = v / 100
						if v ~= ArkInventory.LocationOptionGet( loc_id, { "window", "scale" } ) then
							ArkInventory.LocationOptionSet( loc_id, { "window", "scale" }, v )
							ArkInventory.Frame_Main_Scale_All()
						end
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "slider",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_PADDING"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_PADDING"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_PADDING_TEXT"],
					"min", 4,
					"max", 32,
					"step", 1,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "window", "pad" } )
					end,
					"setFunc", function( v )
						if v ~= ArkInventory.LocationOptionGet( loc_id, { "window", "pad" } ) then
							ArkInventory.LocationOptionSet( loc_id, { "window", "pad" }, v )
							ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
						end
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "slider",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_WIDTH"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_WIDTH"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_WIDTH_TEXT"],
					"min", 8,
					"max", 32,
					"step", 1,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "window", "width" } )
					end,
					"setFunc", function( v )
						if v ~= ArkInventory.LocationOptionGet( loc_id, { "window", "width" } ) then
							ArkInventory.LocationOptionSet( loc_id, { "window", "width" }, v )
							ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
						end
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "color",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BACKGROUND_COLOR"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BACKGROUND_COLOR"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BACKGROUND_COLOR_TEXT"],
					"r", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "background", "r" } ),
					"g", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "background", "g" } ),
					"b", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "background", "b" } ),
					"hasOpacity", true,
					"opacity", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "background", "a" } ),
					"setFunc", function( r, g, b, a )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "background", "r" }, r )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "background", "g" }, g )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "background", "b" }, b )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "background", "a" }, a )
						ArkInventory.Frame_Main_Paint_All()
					end
				)

				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_TEXT"],
					"causesRefresh", true,
					"fullRefresh", true,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "window", "border", "show" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "window", "border", "show" }, not ArkInventory.LocationOptionGet( loc_id, { "window", "border", "show" } ) )
						ArkInventory.Frame_Main_Paint_All()
					end
				)

				w:AddControl(
					"type", "color",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_COLOR"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_COLOR"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BORDER_COLOR_TEXT"],
					"disabled", not ArkInventory.LocationOptionGet( loc_id, { "window", "border", "show" } ),
					"r", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "border", "r" } ),
					"g", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "border", "g" } ),
					"b", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "border", "b" } ),
					"setFunc", function( r, g, b )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "border", "r" }, r )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "border", "g" }, g )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "border", "b" }, b )
						ArkInventory.Frame_Main_Paint_All()
					end
				)

				w:AddControl( "type", "label" )

				w:AddControl(
					"type", "color",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BAGHIGHLIGHT_COLOR"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BAGHIGHLIGHT_COLOR"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_BAGHIGHLIGHT_COLOR_TEXT"],
					"r", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "baghighlight", "r" } ),
					"g", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "baghighlight", "g" } ),
					"b", ArkInventory.LocationOptionGet( loc_id, { "window", "colour", "baghighlight", "b" } ),
					"setFunc", function( r, g, b )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "baghighlight", "r" }, r )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "baghighlight", "g" }, g )
						ArkInventory.LocationOptionSet( loc_id, { "window", "colour", "baghighlight", "b" }, b )
						ArkInventory.Frame_Main_Paint_All()
					end
				)
			
				return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_WINDOW"] .. setting_name )
				
			end

			
			if key == "BAR" then
			
				w:AddControl( "type", "label" )
			
				w:AddControl(
					"type", "label",
					"text", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR"]
				)

				w:AddControl(
					"type", "dropdown",
					"list", anchorpoints,
					"strict", true,
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR_TEXT2"], --string.format( ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR_TEXT2"], ArkInventory.Localise["MOD_MENU_MAIN_WINDOW"], n )
					"getFunc", function()
						local a = ArkInventory.LocationOptionGet( loc_id, { "bar", "anchor" } )
						return string.format( a )
					end,
					"setFunc", function( a )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "anchor" }, tonumber( a ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "slider",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_PER_ROW"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_PER_ROW"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_BARS_PER_ROW_TEXT"],
					"min", 1,
					"max", 16,
					"step", 1,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "bar", "per" } )
					end,
					"setFunc", function( v )
						if v ~= ArkInventory.LocationOptionGet( loc_id, { "bar", "per" } ) then
							ArkInventory.LocationOptionSet( loc_id, { "bar", "per" }, v )
							ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
						end
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "slider",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_PADDING"] .. " - " .. "Internal",
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_PADDING"] .. " - " .. "Internal",
					"tooltipText", "the amount of space to add between the bars and the items",
					"min", 4,
					"max", 32,
					"step", 1,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "bar", "pad", "internal" } )
					end,
					"setFunc", function( v )
						if v ~= ArkInventory.LocationOptionGet( loc_id, { "bar", "pad", "internal" } ) then
							ArkInventory.LocationOptionSet( loc_id, { "bar", "pad", "internal" }, v )
							ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
						end
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "slider",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_PADDING"] .. " - " .. "External",
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_PADDING"] .. " - " .. "External",
					"tooltipText", "the amount of space to add between bars",
					"min", 0,
					"max", 32,
					"step", 1,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "bar", "pad", "external" } )
					end,
					"setFunc", function( v )
						if v ~= ArkInventory.LocationOptionGet( loc_id, { "bar", "pad", "external" } ) then
							ArkInventory.LocationOptionSet( loc_id, { "bar", "pad", "external" }, v )
							ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
						end
					end
				)
				
				w:AddControl( "type", "label" )

				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_COMPACT"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_COMPACT"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_BARS_COMPACT_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "bar", "compact" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "bar", "compact" }, not ArkInventory.LocationOptionGet( loc_id, { "bar", "compact" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
					end
				)

				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_SHOW_EMPTY"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_SHOW_EMPTY"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_BARS_SHOW_EMPTY_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "bar", "showempty" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "bar", "showempty" }, not ArkInventory.LocationOptionGet( loc_id, { "bar", "showempty" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
					end
				)

				w:AddControl(
					"type", "color",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BACKGROUND"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BACKGROUND"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BACKGROUND_TEXT"],
					"r", ArkInventory.LocationOptionGet( loc_id, { "bar", "colour", "background", "r" } ),
					"g", ArkInventory.LocationOptionGet( loc_id, { "bar", "colour", "background", "g" } ),
					"b", ArkInventory.LocationOptionGet( loc_id, { "bar", "colour", "background", "b" } ),
					"hasOpacity", true,
					"opacity", ArkInventory.LocationOptionGet( loc_id, { "bar", "colour", "background", "a" } ),
					"setFunc", function( r, g, b )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "colour", "background", "r" }, r )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "colour", "background", "g" }, g )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "colour", "background", "b" }, b )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "colour", "background", "a" }, a )
						ArkInventory.Frame_Bar_Paint_All()
					end
				)

				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BORDER"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BORDER"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BORDER_TEXT"],
					"causesRefresh", true,
					"fullRefresh", true,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "bar", "border", "show" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "bar", "border", "show" }, not ArkInventory.LocationOptionGet( loc_id, { "bar", "border", "show" } ) )
						ArkInventory.Frame_Bar_Paint_All()
					end
				)
				
				w:AddControl(
					"type", "color",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BORDER_COLOUR"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BORDER_COLOUR"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_BARS_BORDER_COLOUR_TEXT"],
					"disabled", not ArkInventory.LocationOptionGet( loc_id, { "bar", "border", "show" } ),
					"r", ArkInventory.LocationOptionGet( loc_id, { "bar", "colour", "border", "r" } ),
					"g", ArkInventory.LocationOptionGet( loc_id, { "bar", "colour", "border", "g" } ),
					"b", ArkInventory.LocationOptionGet( loc_id, { "bar", "colour", "border", "b" } ),
					"setFunc", function( r, g, b )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "colour", "border", "r" }, r )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "colour", "border", "g" }, g )
						ArkInventory.LocationOptionSet( loc_id, { "bar", "colour", "border", "b" }, b )
						ArkInventory.Frame_Bar_Paint_All()
					end
				)


				return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_BARS"] .. setting_name )
				
			end

			
			if key == "ITEM" then

				w:AddControl( "type", "label" )
			
				w:AddControl(
					"type", "label",
					"text", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR"]
				)

				w:AddControl(
					"type", "dropdown",
					"list", anchorpoints,
					"strict", true,
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR_TEXT2"], --string.format( ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR_TEXT2"], ArkInventory.Localise["MOD_MENU_MAIN_WINDOW"], n )
					"getFunc", function()
						local a = ArkInventory.LocationOptionGet( loc_id, { "slot", "anchor" } )
						return string.format( a )
					end,
					"setFunc", function( a )
						ArkInventory.LocationOptionSet( loc_id, { "slot", "anchor" }, tonumber( a ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
					end
				)

				w:AddControl( "type", "label" )

				w:AddControl(
					"type", "slider",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_PADDING"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_PADDING"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_PADDING_TEXT"],
					"min", 1,
					"max", 16,
					"step", 1,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "pad" } )
					end,
					"setFunc", function( v )
						if v ~= ArkInventory.LocationOptionGet( loc_id, { "slot", "pad" } ) then
							ArkInventory.LocationOptionSet( loc_id, { "slot", "pad" }, v )
							ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
						end
					end
				)
				
				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_BORDER_RARITY"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_BORDER_RARITY"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_BORDER_RARITY_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "border" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "slot", "border" }, not ArkInventory.LocationOptionGet( loc_id, { "slot", "border" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Refresh )
					end
				)
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_HIDDEN"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_HIDDEN"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_HIDDEN_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "ignorehidden" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "slot", "ignorehidden" }, not ArkInventory.LocationOptionGet( loc_id, { "slot", "ignorehidden" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
					end
				)
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_FADE"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_FADE"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_ITEMS_FADE_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "offline", "fade" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "slot", "offline", "fade" }, not ArkInventory.LocationOptionGet( loc_id, { "slot", "offline", "fade" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Refresh )
					end
				)

				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "label",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM"]
				)
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_SHOW"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_SHOW"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_SHOW_TEXT"],
					"causesRefresh", true,
					"fullRefresh", true,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "new", "show" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "slot", "new", "show" }, not ArkInventory.LocationOptionGet( loc_id, { "slot", "new", "show" } ) )
						ArkInventory.NewItemReset( loc_id )
						ArkInventory.NewItemIndicator( loc_id )
					end
				)
				
				w:AddControl(
					"type", "color",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_COLOUR"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_COLOUR"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_COLOUR_TEXT"],
					"disabled", not ArkInventory.LocationOptionGet( loc_id, { "slot", "new", "show" } ),
					"r", ArkInventory.LocationOptionGet( loc_id, { "slot", "new", "colour", "r" } ),
					"g", ArkInventory.LocationOptionGet( loc_id, { "slot", "new", "colour", "g" } ),
					"b", ArkInventory.LocationOptionGet( loc_id, { "slot", "new", "colour", "b" } ),
					"setFunc", function( r, g, b )
						ArkInventory.LocationOptionSet( loc_id, { "slot", "new", "colour", "r" }, r )
						ArkInventory.LocationOptionSet( loc_id, { "slot", "new", "colour", "g" }, g )
						ArkInventory.LocationOptionSet( loc_id, { "slot", "new", "colour", "b" }, b )
						ArkInventory.NewItemIndicator( loc_id )
					end
				)

				w:AddControl(
					"type", "button",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_RESET"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_RESET"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_NEWITEM_RESET_TEXT"],
					"disabled", not ArkInventory.LocationOptionGet( loc_id, { "slot", "new", "show" } ),
					"execFunc", function()
						ArkInventory.NewItemReset( loc_id )
						ArkInventory.NewItemIndicator( loc_id )
					end
				)
				
				return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_ITEMS"] .. setting_name )
				
			end

			
			if key == "SORT" then
			
				w:AddControl( "type", "label" )
			
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INSTANT"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INSTANT"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INSTANT_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "sort", "instant" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "sort", "instant" }, not ArkInventory.LocationOptionGet( loc_id, { "sort", "instant" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Resort )
					end
				)

				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_NAME_REVERSE"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_NAME_REVERSE"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_NAME_REVERSE_TEXT"],
					"disabled", ArkInventory.LocationOptionGet( loc_id, { "sort", "bagslot" } ) or not ArkInventory.LocationOptionGet( loc_id, { "sort", "name" } ),
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "sort", "reversed" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "sort", "reversed" }, not ArkInventory.LocationOptionGet( loc_id, { "sort", "reversed" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Resort )
					end
				)
				
				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_BAGSLOT"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_BAGSLOT"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_BAGSLOT_TEXT"],
					"isRadio", true,
					"causesRefresh", true,
					"fullRefresh", true,
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "sort", "bagslot" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "sort", "bagslot" }, not ArkInventory.LocationOptionGet( loc_id, { "sort", "bagslot" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Resort )
					end
				)
			
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_USER"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_USER"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_USER_TEXT"],
					"isRadio", true,
					"causesRefresh", true,
					"fullRefresh", true,
					"getFunc", function()
						return not ArkInventory.LocationOptionGet( loc_id, { "sort", "bagslot" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "sort", "bagslot" }, not ArkInventory.LocationOptionGet( loc_id, { "sort", "bagslot" } ) )
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Resort )
					end
				)
				
				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "label",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_ORDER"]
				)
			
				local t = ArkInventory.LocationOptionGet( loc_id, { "sortorder" } )
				for k, v in ipairs( t ) do
						
					if ArkInventory.Const.SortKeys[v] then
							
						local checked = ArkInventory.LocationOptionGet( loc_id, { "sort", v } )
						local disabled = ArkInventory.LocationOptionGet( loc_id, { "sort", "bagslot" } )
						local n = ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_" .. string.upper( v )]
						local tt_text = ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_" .. string.upper( v ) .. "_TEXT"]

						if not disabled then
							if checked then
								n = GREEN_FONT_COLOR_CODE .. n .. FONT_COLOR_CODE_CLOSE
							else
								n = RED_FONT_COLOR_CODE .. n .. FONT_COLOR_CODE_CLOSE
								tt_text = tt_text .. "\n\n" .. RED_FONT_COLOR_CODE .. ArkInventory.Localise["MOD_MENU_MAIN_SORTING_NOT_INCLUDED"]
							end
						end
							
						w:AddControl(
							"type", "checkbox",
							"text", k .. ". " .. n,
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_" .. string.upper( v )],
							"tooltipText", tt_text,
							"disabled", disabled,
							"causesRefresh", true,
							"fullRefresh", true,
							"getFunc", function()
								return checked
							end,
							"setFunc", function()
								ArkInventory.LocationOptionSet( loc_id, { "sort", v }, not checked )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Resort )
							end
						)

						w:AddControl(
							"type", "button",
							"text", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_MOVE_UP"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_MOVE_UP"],
							"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SORTING_MOVE_UP_TEXT"], LIGHTYELLOW_FONT_COLOR_CODE .. ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_" .. string.upper( v )] .. FONT_COLOR_CODE_CLOSE ),
							"disabled", disabled or not checked,
							"causesRefresh", true,
							"fullRefresh", true,
							"execFunc", function()
								ArkInventory.PrintPlus( { "ArkInventory.ItemSortKeyMoveUp( ", loc_id, ", ", v, " )" } )
								ArkInventory.ItemSortKeyMoveUp( loc_id, v )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Resort )
							end
						)

						w:AddControl(
							"type", "button",
							"text", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_MOVE_DOWN"],
							"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SORTING_MOVE_DOWN"],
							"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SORTING_MOVE_DOWN_TEXT"], LIGHTYELLOW_FONT_COLOR_CODE .. ArkInventory.Localise["MOD_MENU_MAIN_SORTING_INCLUDE_" .. string.upper( v )] .. FONT_COLOR_CODE_CLOSE ),
							"disabled", disabled or not checked,
							"causesRefresh", true,
							"fullRefresh", true,
							"execFunc", function()
								ArkInventory.PrintPlus( { "ArkInventory.ItemSortKeyMoveDown( ", loc_id, ", ", v, " )" } )
								ArkInventory.ItemSortKeyMoveDown( loc_id, v )
								ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Resort )
							end
						)
						
					end
					
				end

			
				return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_SORTING"] .. setting_name )
				
			end

			
			if key == "EMPTY" then
			
				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_ICON"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_ICON"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_ICON_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "empty", "icon" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "slot", "empty", "icon" }, not ArkInventory.LocationOptionGet( loc_id, { "slot", "empty", "icon" } ) )
						ArkInventory.UiItem_Empty()
					end
				)
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_BORDER"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_BORDER"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_BORDER_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "empty", "border" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "slot", "empty", "border" }, not ArkInventory.LocationOptionGet( loc_id, { "slot", "empty", "border" } ) )
						ArkInventory.UiItem_Empty()
					end
				)
				
				w:AddControl(
					"type", "checkbox",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_CLUMP"],
					"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_CLUMP"],
					"tooltipText", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_CLUMP_TEXT"],
					"getFunc", function()
						return ArkInventory.LocationOptionGet( loc_id, { "slot", "empty", "clump" } )
					end,
					"setFunc", function()
						ArkInventory.LocationOptionSet( loc_id, { "slot", "empty", "clump" }, not ArkInventory.LocationOptionGet( loc_id, { "slot", "empty", "clump" } ) )
						ArkInventory.ItemCacheClear()
						ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
					end
				)
				
				w:AddControl( "type", "label" )
				
				w:AddControl(
					"type", "label",
					"text", ArkInventory.Localise["MOD_MENU_MAIN_EMPTY_COLOUR"]
				)
				
				for k, v in ArkInventory.spairs( ArkInventory.LocationOptionGet( loc_id, { "slot", "data" } ), function(a,b) return ArkInventory.LocationOptionGet( loc_id, { "slot", "data", a, "long" } ) < ArkInventory.LocationOptionGet( loc_id, { "slot", "data", b, "long" } ) end ) do
					w:AddControl(
						"type", "color",
						"text", v.long,
						--"tooltipTitle", ArkInventory.Localise[""],
						--"tooltipText", ArkInventory.Localise[""],
						"r", ArkInventory.LocationOptionGet( loc_id, { "slot", "data", k, "colour", "r" } ),
						"g", ArkInventory.LocationOptionGet( loc_id, { "slot", "data", k, "colour", "g" } ),
						"b", ArkInventory.LocationOptionGet( loc_id, { "slot", "data", k, "colour", "b" } ),
						"setFunc", function( r, g, b )
							ArkInventory.LocationOptionSet( loc_id, { "slot", "data", k, "colour", "r" }, r )
							ArkInventory.LocationOptionSet( loc_id, { "slot", "data", k, "colour", "g" }, g )
							ArkInventory.LocationOptionSet( loc_id, { "slot", "data", k, "colour", "b" }, b )
							ArkInventory.UiItem_Empty()
						end
					)
				end

				return string.upper( ArkInventory.Localise["MOD_MENU_MAIN_EMPTY"] .. setting_name )
				
			end
			


			w:AddControl( "type", "label" )
			
			w:AddControl(
				"type", "label",
				"text", ArkInventory.Localise["MOD_MENU_MAIN_SETTINGS"]
			)

			w:AddControl(
				"type", "dropdown",
				"list", locations,
				"strict", true,
				"causesRefresh", true,
				"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_SETTINGS"],
				"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_SETTINGS_LOCATION_TEXT"], ArkInventory.Global.Location[ArkInventory.db.profile.option.use[loc_id]].Name, ArkInventory.Global.Location[loc_id].Name ),
				"getFunc", function()
					return string.format( ArkInventory.db.profile.option.use[loc_id] )
				end,
				"setFunc", function( set_id )
					ArkInventory.db.profile.option.use[loc_id] = tonumber( set_id )
					ArkInventory.Frame_Main_Generate( nil, ArkInventory.Const.Window.Draw.Recalculate )
				end
			)
			
			w:AddControl( "type", "label" )
			
			w:AddControl(
				"type", "label",
				"text", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR"]
			)

			w:AddControl(
				"type", "dropdown",
				"list", anchorpoints,
				"strict", true,
				"tooltipTitle", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR"],
				"tooltipText", ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR_TEXT2"], --string.format( ArkInventory.Localise["MOD_MENU_GENERIC_ANCHOR_TEXT2"], ArkInventory.Localise["MOD_MENU_MAIN_WINDOW"], n )
				"getFunc", function()
					local a = ArkInventory.LocationOptionGet( loc_id, { "anchor", loc_id, "point" } )
					return string.format( a )
				end,
				"setFunc", function( a )
					ArkInventory.LocationOptionSet( loc_id, { "anchor", loc_id, "point" }, tonumber( a ) )
					ArkInventory.Frame_Main_Anchor_Set( loc_id )
				end
			)

			w:AddControl( "type", "label" )
				
			w:AddControl(
				"type", "checkbox",
				"text", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_ANCHOR_LOCK"],
				"tooltipTitle", ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_ANCHOR_LOCK"],
				"tooltipText", string.format( ArkInventory.Localise["MOD_MENU_MAIN_WINDOW_ANCHOR_LOCK_TEXT"], ArkInventory.Global.Location[loc_id].Name ),
				"getFunc", function()
					return ArkInventory.LocationOptionGet( loc_id, { "anchor", loc_id, "locked" } )
				end,
				"setFunc", function()
					ArkInventory.LocationOptionSet( loc_id, { "anchor", loc_id, "locked" }, not ArkInventory.LocationOptionGet( loc_id, { "anchor", loc_id, "locked" } ) )
					ArkInventory.Frame_Main_Anchor_Set( loc_id )
				end
			)
			
			
			
			
			
			
			return string.upper( ArkInventory.Global.Location[loc_id].Name )
		
		end
	
	
	
		return ""
	
	end

	
	
	return ""

end
